package cn.iocoder.yudao.module.member.mq.consumer.score.core;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.ArrayUtil;
import cn.hutool.json.JSONUtil;
import cn.iocoder.boot.module.order.api.OrderApi;
import cn.iocoder.boot.module.order.api.dto.OrderObjectiveApiDO;
import cn.iocoder.boot.module.order.api.dto.OrderRespDTO;
import cn.iocoder.boot.module.order.enums.OrderStatusApiEnum;
import cn.iocoder.yudao.module.customer.dal.dataobject.customerContacts.CustomerContactsDO;
import cn.iocoder.yudao.module.customer.service.customerContacts.CustomerContactsService;
import cn.iocoder.yudao.module.member.api.score.MemberUserScoreApi;
import cn.iocoder.yudao.module.member.api.score.dto.MemberUserScoreOperateReqDTO;
import cn.iocoder.yudao.module.member.dal.dataobject.scoreRule.ScoreRuleDO;
import cn.iocoder.yudao.module.member.enums.*;
import cn.iocoder.yudao.module.member.mq.message.ScoreMessage;
import cn.iocoder.yudao.module.member.service.scoreLog.MemberUserScoreLogService;
import cn.iocoder.yudao.module.member.service.scoreRule.ScoreRuleService;
import cn.iocoder.yudao.module.member.service.user.MemberUserService;
import cn.iocoder.yudao.module.member.vo.scoreRule.extra.ScoreRuleOrderVExtraVO;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.HashMap;
import java.util.List;
import java.util.Objects;

/**
 * @author zhangfeng
 */
@Service
@Slf4j
public class OrderVStrategy extends AbstractScoreRuleStrategy {
    @Resource
    private OrderApi orderApi;
    @Resource
    private MemberUserScoreApi memberUserScoreApi;
    @Resource
    private CustomerContactsService customerContactsService;

    public OrderVStrategy(ScoreRuleService scoreRuleService, MemberUserScoreLogService memberUserScoreLogService, MemberUserService memberUserService) {
        super(scoreRuleService, memberUserScoreLogService, memberUserService);
    }

    @Override
    public void addScore(ScoreMessage message) {
        log.info("Received order V message,orderID:{}", message.getOrderId());
        Long orderId = message.getOrderId();
        OrderRespDTO order;
        try {
            order = orderApi.getOrder(orderId, "");
        } catch (Exception e) {
            log.error("Order in shipping listener: Get order error,orderId:{}", orderId, e);
            return;
        }
        if (Objects.isNull(order)) {
            log.error("Order in shipping listener: The order does not exist,orderId:{}", orderId);
            return;
        }
        // 订单状态应为起运
        if (!Objects.equals(order.getStatus(), OrderStatusApiEnum.IN_SHIPPING.getValue())) {
            return;
        }
        // 判断海运空运
        Integer transportId = order.getTransportId();
        ScoreRuleDO scoreRuleDO = scoreRuleService.getEnableScoreRuleByType(ScoreRuleTypeEnum.ORDER_V, TransportTypeEnum.parseByValue(transportId));
        if (Objects.isNull(scoreRuleDO)) {
            log.info("Order in shipping listener: No score rule match,orderID:{}", orderId);
            return;
        }
        // 目的地信息
        OrderObjectiveApiDO orderObjectiveDO = order.getOrderObjectiveDO();
        ScoreRuleOrderVExtraVO extraInfo = JSONUtil.toBean(scoreRuleDO.getExtra(), ScoreRuleOrderVExtraVO.class);
        // 提货点是否包含
        String[] warehouseIds = extraInfo.getReceiveAddr().split(",");
        if (!ArrayUtil.contains(warehouseIds, orderObjectiveDO.getObjectiveWarehouseId().toString())) {
            return;
        }
        // 计算积分
        List<ScoreRuleOrderVExtraVO.OrderVRule> orderVRules = extraInfo.getOrderVRule();
        Integer scoreCount = 0;
        if (transportId == TransportTypeEnum.OCEAN_LCL.getValue()) {
            // 海运算重量
            BigDecimal orgVWeight = order.getOrgVWeight();
            scoreCount = computeScoreCount(orgVWeight, orderVRules);
        } else if (transportId == TransportTypeEnum.SPECIAL_LINE_AIR_FREIGHT.getValue()) {
            // 空运校验渠道
            // 渠道id
            Long channelId = order.getChannelId();
            String[] channels = extraInfo.getChannel().split(",");
            if (!ArrayUtil.contains(channels, channelId.toString())) {
                return;
            }
            // 空运算体积
            BigDecimal orgWVolume = order.getOrgWVolume();
            scoreCount = computeScoreCount(orgWVolume, orderVRules);
        }
        if (scoreCount <= 0) {
            log.info("Order in shipping listener: The score count is less than or equal to 0,orderId:{}", orderId);
            return;
        }

        // 会员id
        Long userId = order.getUserId();
        // 订单入口
        String orderEntryString = extraInfo.getOrderEntry();
        if (StringUtils.isBlank(orderEntryString)) {
            log.info("Order in shipping listener: orderEntry is null,scoreRuleID:{}", scoreRuleDO.getId());
            return;
        }
        String[] orderEntry = orderEntryString.split(",");
        if (userId <= 0) {
            // 后台下单
            if (!ArrayUtil.contains(orderEntry, String.valueOf(PlatformTypeEnum.BACKEND.getValue()))) {
                log.info("Order in shipping listener: The order is placed by the background, userId:{}", userId);
                return;
            }
            // 查询客户绑定会员
            List<CustomerContactsDO> customerContacts = customerContactsService.getCustomerContactsListByCustomerId(order.getCustomerId());
            if (CollectionUtil.isEmpty(customerContacts)) {
                log.info("Order in shipping listener: The customer is not bound to the member, customerId:{}", order.getCustomerId());
                return;
            }
            for (CustomerContactsDO customerContact : customerContacts) {
                if (customerContact.getIsDefault() == 1 && customerContact.getUserid() != null) {
                    userId = customerContact.getUserid();
                }
            }
            if (userId <= 0) {
                log.info("Order in shipping listener: The customer is not have default userId, customerId:{}", order.getCustomerId());
                return;
            }
        } else if (!ArrayUtil.contains(orderEntry, String.valueOf(PlatformTypeEnum.APP.getValue())) && !ArrayUtil.contains(orderEntry, String.valueOf(PlatformTypeEnum.WEB.getValue()))) {
            log.info("Order in shipping listener: The order is placed by the app or web, userId:{}", userId);
            return;
        }
        // 是否首单，首单只加一次
        if (extraInfo.getFirstOrder() == YesOrNoTypeEnum.YES.ordinal()) {
            if (!orderApi.isFirstOrder(userId)) {
                log.info("Order in shipping listener: Not first order，userId:{}", userId);
                return;
            }
        }
        // 校验累计最高分
        Integer userScoreTotalCount = getUserScoreTotalCount(scoreRuleDO.getId(), userId);
        if (userScoreTotalCount >= scoreRuleDO.getMaxScoreTotal()) {
            log.info("Order in shipping listener: The user has reached the maximum score,userId:{},scoreRuleId:{}", userId, scoreRuleDO.getId());
            return;
        }
        //if (userScoreTotalCount + scoreCount > scoreRuleDO.getMaxScoreTotal()) {
        //    scoreCount = scoreRuleDO.getMaxScoreTotal() - userScoreTotalCount;
        //}
        // 增加积分
        try {
            HashMap<String, Object> map = new HashMap<>();
            map.put("orderId", orderId);
            memberUserScoreApi.operateScore(MemberUserScoreOperateReqDTO.builder()
                    .memberId(userId)
                    .scoreCount(scoreCount)
                    .sourceType(ScoreSourceTypeEnum.ORDER_V)
                    .ruleId(scoreRuleDO.getId())
                    .expireDays(scoreRuleDO.getScorePeriod())
                    .releationId(String.valueOf(orderId))
                    .uniqueId(String.valueOf(orderId))
                    .extParam(map)
                    .build()
            );
        } catch (Exception e) {
            log.error("Order in shipping listener: operateScore error", e);
        }
        log.info("Order V rule add score success,ruleID:{},orderID:{},userID:{}", scoreRuleDO.getId(), message.getOrderId(), userId);
    }

    @Override
    public ScoreRuleTypeEnum getStrategyScoreRuleType() {
        return ScoreRuleTypeEnum.ORDER_V;
    }


    /**
     * 根据规则计算获取积分
     *
     * @param weightOrVolume 重量或体积
     * @param orderVRules    规则
     * @return
     */
    private Integer computeScoreCount(BigDecimal weightOrVolume, List<ScoreRuleOrderVExtraVO.OrderVRule> orderVRules) {
        Integer scoreCount = 0;
        int w = weightOrVolume.setScale(0, RoundingMode.HALF_UP).intValue();
        ScoreRuleOrderVExtraVO.OrderVRule lastRule = orderVRules.get(orderVRules.size() - 1);
        if (w >= lastRule.getHigh()) {
            scoreCount = lastRule.getScore();
        }
        for (int i = orderVRules.size() - 1; i >= 0; i--) {
            ScoreRuleOrderVExtraVO.OrderVRule orderVRule = orderVRules.get(i);
            if (orderVRule.getLow() <= w && orderVRule.getHigh() >= w) {
                scoreCount = orderVRule.getScore();
                break;
            }
        }
        return scoreCount;
    }
}
