package cn.iocoder.yudao.module.member.service.score;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.mybatis.core.service.AbstractService;
import cn.iocoder.yudao.module.member.api.score.dto.MemberUserScoreUpdateReqDTO;
import cn.iocoder.yudao.module.member.dal.dataobject.score.MemberUserScoreDO;
import cn.iocoder.yudao.module.member.dal.mysql.memberUserScore.MemberUserScoreMapper;
import cn.iocoder.yudao.module.member.enums.ScoreOperateTypeEnum;
import cn.iocoder.yudao.module.member.enums.ScoreSourceTypeEnum;
import cn.iocoder.yudao.module.member.vo.memberUserScore.MemberUserScoreBackVO;
import cn.iocoder.yudao.module.member.vo.memberUserScore.MemberUserScoreQueryVO;
import org.springframework.context.annotation.Lazy;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;
import java.util.Collection;
import java.util.Date;
import java.util.List;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.member.enums.ErrorCodeConstants.MEMBER_ID_IS_NULL;
import static cn.iocoder.yudao.module.member.enums.ErrorCodeConstants.MEMBER_SCORE_NOT_ENOUGH;

/**
 * 会员积分 Service 实现类
 *
 * @author 系统管理员
 */
@Service
@Validated
@Lazy
public class MemberUserScoreServiceImpl extends AbstractService<MemberUserScoreMapper, MemberUserScoreDO> implements MemberUserScoreService {

    @Resource
    private MemberUserScoreMapper userScoreMapper;

    @Override
    public void deleteUserScore(Long id) {
        // 删除
        userScoreMapper.deleteById(id);
    }

    @Override
    public MemberUserScoreDO getUserScore(Long id) {
        return userScoreMapper.selectById(id);
    }

    @Override
    public List<MemberUserScoreDO> getUserScoreList(Collection<Long> ids) {
        return userScoreMapper.selectBatchIds(ids);
    }

    @Override
    public PageResult<MemberUserScoreBackVO> getUserScorePage(MemberUserScoreQueryVO query) {
        int start = (query.getPageNo() - 1) * query.getPageSize();
        int size = query.getPageSize();
        List<MemberUserScoreBackVO> list = userScoreMapper.getUserScoreList(start, size, query);
        int total = userScoreMapper.countUserScore(query);
        return new PageResult<>(list, total, query.getPageSize(), query.getPageNo(), (total + query.getPageSize() - 1) / query.getPageSize());
    }

    @Override
    public List<MemberUserScoreDO> getUserScoreList(MemberUserScoreQueryVO query) {
        return userScoreMapper.selectList(query);
    }

    @Override
    public void updateUserScore(MemberUserScoreUpdateReqDTO query) {
        // 增加积分，只需要增加持有积分即可
        // 减少积分，持有积分减少， 已兑换或者已失效积分增加
        if (query.getMemberId() == null) {
            throw exception(MEMBER_ID_IS_NULL);
        }
        MemberUserScoreDO memberUserScoreDO = userScoreMapper.getByMemberId(query.getMemberId());
        if (memberUserScoreDO == null) {
            memberUserScoreDO = initMemberUserScore(query.getMemberId());
        }
        // 回退流程增加积分
        if (query.getSourceType().getReverseSource() != null && query.getOperateType() == ScoreOperateTypeEnum.ADD) {
            // 回退流程增加积分，不仅需要新增持有积分，还需要减少使用积分
            memberUserScoreDO.setHoldScore(memberUserScoreDO.getHoldScore() + query.getScoreCount());
            memberUserScoreDO.setUsedScore(memberUserScoreDO.getUsedScore() - query.getScoreCount());
            this.saveOrUpdate(memberUserScoreDO);
            return;
        }
        //正常流程 增加积分 减少积分
        if (query.getOperateType() == ScoreOperateTypeEnum.ADD) {
            memberUserScoreDO.setHoldScore(memberUserScoreDO.getHoldScore() + query.getScoreCount());
            this.saveOrUpdate(memberUserScoreDO);
        }
        if (query.getOperateType() == ScoreOperateTypeEnum.REDUCE) {
            if (memberUserScoreDO.getHoldScore() < query.getScoreCount()) {
                throw exception(MEMBER_SCORE_NOT_ENOUGH);
            }
            memberUserScoreDO.setHoldScore(memberUserScoreDO.getHoldScore() - query.getScoreCount());
            if (ScoreSourceTypeEnum.getInvalidScoreStatusSourceType().contains(query.getSourceType())) {
                memberUserScoreDO.setExpiredScore(memberUserScoreDO.getExpiredScore() + query.getScoreCount());
            } else {
                memberUserScoreDO.setUsedScore(memberUserScoreDO.getUsedScore() + query.getScoreCount());
            }
            this.saveOrUpdate(memberUserScoreDO);
        }
    }

    private MemberUserScoreDO initMemberUserScore(Long memberId) {
        MemberUserScoreDO memberUserScoreDO = new MemberUserScoreDO();
        memberUserScoreDO.setMemberId(memberId);
        memberUserScoreDO.setHoldScore(0);
        memberUserScoreDO.setExpiredScore(0);
        memberUserScoreDO.setUsedScore(0);
        memberUserScoreDO.setCreateTime(new Date());
        return memberUserScoreDO;
    }
}
