package cn.iocoder.yudao.module.member.service.scoreRule;

import java.time.Instant;
import java.util.*;
import javax.annotation.Resource;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.json.JSONUtil;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQuery;
import cn.iocoder.yudao.framework.mybatis.core.service.AbstractService;
import cn.iocoder.yudao.module.member.convert.scoreRule.ScoreRuleConvert;
import cn.iocoder.yudao.module.member.dal.dataobject.scoreRule.ScoreRuleDO;
import cn.iocoder.yudao.module.member.dal.mysql.scoreRule.ScoreRuleMapper;
import cn.iocoder.yudao.module.member.dal.redis.scoreRule.ScoreRuleRedisDao;
import cn.iocoder.yudao.module.member.enums.ScoreRuleStatusEnum;
import cn.iocoder.yudao.module.member.enums.ScoreRuleTypeEnum;
import cn.iocoder.yudao.module.member.enums.TransportTypeEnum;
import cn.iocoder.yudao.module.member.enums.YesOrNoTypeEnum;
import cn.iocoder.yudao.module.member.vo.scoreRule.*;
import cn.iocoder.yudao.module.member.vo.scoreRule.extra.ScoreRuleOrderVExtraVO;
import cn.iocoder.yudao.module.member.vo.scoreRule.extra.ScoreRuleRegisterExtraVO;
import cn.iocoder.yudao.module.member.vo.scoreRule.extra.ScoreRuleShareExtraVO;
import cn.iocoder.yudao.module.member.vo.scoreRule.extra.ScoreRulerRecommendExtraVO;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.member.enums.ErrorCodeConstants.*;

/**
 * 积分规则 Service 实现类
 *
 * @author 系统管理员
 */
@Service
@Validated
public class ScoreRuleServiceImpl extends AbstractService<ScoreRuleMapper, ScoreRuleDO> implements ScoreRuleService {

    @Resource
    private ScoreRuleMapper scoreRuleMapper;
    @Resource
    private ScoreRuleRedisDao scoreRuleRedisDao;

    @Override
    public Long createScoreRule(ScoreRuleCreateReqVO createReqVO) {
        ScoreRuleDO scoreRule = ScoreRuleConvert.INSTANCE.convert(createReqVO);
        if (scoreRule.getStatus() != ScoreRuleStatusEnum.DISABLED.getValue() && scoreRule.getStatus() != ScoreRuleStatusEnum.ENABLED.getValue()) {
            throw exception(SCORE_RULE_FIELD_ERROR);
        }
        verifyCommon(createReqVO);
        Integer scoreRuleType = scoreRule.getType();
        verifyAndSetExtraDO(createReqVO, scoreRuleType, scoreRule);
        scoreRuleMapper.insert(scoreRule);
        scoreRuleRedisDao.deleteEnableScoreRule(scoreRule.getType());
        // 返回
        return scoreRule.getId();
    }

    private void verifyCommon(ScoreRuleBaseVO scoreRuleIn) {
        //校验公共入参
        if (scoreRuleIn.getType() != ScoreRuleTypeEnum.ORDER_V.getValue() && scoreRuleIn.getGetScoreOnce() <= 0) {
            throw exception(SCORE_RULE_FIELD_ERROR);
        }
        if (scoreRuleIn.getType() != ScoreRuleTypeEnum.RECOMMEND.getValue() && scoreRuleIn.getMaxScoreTotal() <= 0) {
            throw exception(SCORE_RULE_FIELD_ERROR);
        }
        if (scoreRuleIn.getStartTime().after((scoreRuleIn.getEndTime())) || scoreRuleIn.getEndTime().before(Date.from(Instant.now()))) {
            throw exception(SCORE_RULE_FIELD_ERROR);
        }
        if (scoreRuleIn.getScorePeriod() <= 0) {
            throw exception(SCORE_RULE_FIELD_ERROR);
        }
        //如果是启用，校验同一个规则下，有效期内有没有重复的规则设置，分享不校验。海运空运分开算
        if (scoreRuleIn.getStatus() == ScoreRuleStatusEnum.ENABLED.getValue() && scoreRuleIn.getType() != ScoreRuleTypeEnum.SHARE.getValue()) {
            LambdaQuery<ScoreRuleDO> scoreRuleDOLambdaQuery = new LambdaQuery<>();
            scoreRuleDOLambdaQuery.eq(ScoreRuleDO::getStatus, ScoreRuleStatusEnum.ENABLED.getValue())
                    .eq(ScoreRuleDO::getType, scoreRuleIn.getType());
            if (scoreRuleIn.getType() == ScoreRuleTypeEnum.ORDER_V.getValue()) {
                ScoreRuleOrderVExtraVO extraOrderV = scoreRuleIn.getExtraOrderV();
                if (extraOrderV == null) {
                    throw exception(SCORE_RULE_FIELD_ERROR);
                }
                Integer transportType = extraOrderV.getTransportType();
                if (transportType != TransportTypeEnum.OCEAN_LCL.getValue() && transportType != TransportTypeEnum.SPECIAL_LINE_AIR_FREIGHT.getValue()) {
                    throw exception(SCORE_RULE_FIELD_ERROR);
                }
                List<ScoreRuleDO> scoreRuleDOS = scoreRuleMapper.selectList(scoreRuleDOLambdaQuery);
                if (scoreRuleDOS != null && !scoreRuleDOS.isEmpty()) {
                    for (ScoreRuleDO scoreRuleDO : scoreRuleDOS) {
                        ScoreRuleOrderVExtraVO bean = JSONUtil.toBean(scoreRuleDO.getExtra(), ScoreRuleOrderVExtraVO.class);
                        if (Objects.equals(bean.getTransportType(), transportType)) {
                            throw exception(SCORE_RULE_FIELD_ERROR);
                        }
                    }
                }
            } else {
                Long count = scoreRuleMapper.selectCount(scoreRuleDOLambdaQuery);
                if (count > 0) {
                    throw exception(SCORE_RULE_FIELD_ERROR);
                }
            }
        }
    }


    @Override
    public void updateScoreRule(ScoreRuleUpdateReqVO updateReqVO) {
        ScoreRuleDO scoreRuleDO = scoreRuleMapper.selectById(updateReqVO.getId());
        if (scoreRuleDO == null) {
            throw exception(SCORE_RULE_NOT_EXISTS);
        }
        if (scoreRuleDO.getStatus() != ScoreRuleStatusEnum.DISABLED.getValue()) {
            throw exception(SCORE_RULE_UPDATE_ERROR);
        }
        ScoreRuleDO scoreRule = ScoreRuleConvert.INSTANCE.convert(updateReqVO);
        verifyCommon(updateReqVO);
        verifyAndSetExtraDO(updateReqVO, updateReqVO.getType(), scoreRule);
        // 更新
        scoreRuleMapper.updateById(scoreRule);
        scoreRuleRedisDao.deleteEnableScoreRule(scoreRule.getType());
    }

    /**
     * 设置额外信息
     *
     * @param reqVO         请求实体
     * @param scoreRuleType 指标类型
     * @param scoreRule     需要填充额外字段的DO
     */
    private void verifyAndSetExtraDO(ScoreRuleBaseVO reqVO, Integer scoreRuleType, ScoreRuleDO scoreRule) {
        if (scoreRuleType == ScoreRuleTypeEnum.ORDER_V.getValue()) {
            ScoreRuleOrderVExtraVO extraOrderV = reqVO.getExtraOrderV();
            if (extraOrderV == null) {
                throw exception(SCORE_RULE_FIELD_ERROR);
            }
            if (extraOrderV.getFirstOrder() != YesOrNoTypeEnum.YES.ordinal() && extraOrderV.getFirstOrder() != YesOrNoTypeEnum.NO.ordinal()) {
                throw exception(SCORE_RULE_FIELD_ERROR);
            }
            if (extraOrderV.getTransportType() != TransportTypeEnum.OCEAN_LCL.getValue() && extraOrderV.getTransportType() != TransportTypeEnum.SPECIAL_LINE_AIR_FREIGHT.getValue()) {
                throw exception(SCORE_RULE_FIELD_ERROR);
            }
            if (StringUtils.isAnyBlank(extraOrderV.getTargetCountry(), extraOrderV.getTargetCity(), extraOrderV.getReceiveAddr(), extraOrderV.getOrderEntry())) {
                throw exception(SCORE_RULE_FIELD_ERROR);
            }
            verifyOrderVRule(extraOrderV.getOrderVRule());
            scoreRule.setExtra(JSONUtil.toJsonStr(extraOrderV));
        } else if (scoreRuleType == ScoreRuleTypeEnum.REGISTER.getValue()) {
            if (reqVO.getExtraRegister() == null) {
                throw exception(SCORE_RULE_FIELD_ERROR);
            }
            if (StringUtils.isBlank(reqVO.getExtraRegister().getRegisterPlatform())) {
                throw exception(SCORE_RULE_FIELD_ERROR);
            }
            scoreRule.setExtra(JSONUtil.toJsonStr(reqVO.getExtraRegister()));
        } else if (scoreRuleType == ScoreRuleTypeEnum.RECOMMEND.getValue()) {
            ScoreRulerRecommendExtraVO extraRecommend = reqVO.getExtraRecommend();
            if (extraRecommend == null) {
                throw exception(SCORE_RULE_FIELD_ERROR);
            }
            if (extraRecommend.getShareStatus() != YesOrNoTypeEnum.YES.ordinal() && extraRecommend.getShareStatus() != YesOrNoTypeEnum.NO.ordinal()) {
                throw exception(SCORE_RULE_FIELD_ERROR);
            }
            if (StringUtils.isAnyBlank(extraRecommend.getShareContentEn(), extraRecommend.getShareContentZh())) {
                throw exception(SCORE_RULE_FIELD_ERROR);
            }
            scoreRule.setExtra(JSONUtil.toJsonStr(extraRecommend));
        } else if (scoreRuleType == ScoreRuleTypeEnum.SHARE.getValue()) {
            ScoreRuleShareExtraVO extraShare = reqVO.getExtraShare();
            if (extraShare == null) {
                throw exception(SCORE_RULE_FIELD_ERROR);
            }
            if (StringUtils.isAnyBlank(extraShare.getActivityDescZh(), extraShare.getActivityDescEn(), extraShare.getActivityUrl())) {
                throw exception(SCORE_RULE_FIELD_ERROR);
            }
            if (extraShare.getShareStatus() != YesOrNoTypeEnum.YES.ordinal() && extraShare.getShareStatus() != YesOrNoTypeEnum.NO.ordinal()) {
                throw exception(SCORE_RULE_FIELD_ERROR);
            }
            scoreRule.setExtra(JSONUtil.toJsonStr(extraShare));
        } else {
            throw exception(SCORE_RULE_NOT_EXISTS);
        }
    }

    private void verifyOrderVRule(List<ScoreRuleOrderVExtraVO.OrderVRule> extraOrderVRule) {
        if (extraOrderVRule == null || extraOrderVRule.isEmpty()) {
            throw exception(SCORE_RULE_FIELD_ERROR);
        }
        int low = 0;
        for (ScoreRuleOrderVExtraVO.OrderVRule orderVRule : extraOrderVRule) {
            if (orderVRule.getLow() < low || orderVRule.getHigh() <= orderVRule.getLow() || orderVRule.getScore() <= 0) {
                throw exception(SCORE_RULE_FIELD_ERROR);
            }
            low = orderVRule.getHigh();
        }
    }

    @Override
    public void deleteScoreRule(Long id) {
        ScoreRuleDO scoreRuleDO = scoreRuleMapper.selectById(id);
        if (scoreRuleDO == null) {
            throw exception(SCORE_RULE_NOT_EXISTS);
        }
        if (scoreRuleDO.getStatus() != ScoreRuleStatusEnum.DISABLED.getValue()) {
            throw exception(SCORE_RULE_DELETE_ERROR);
        }
        // 删除
        scoreRuleMapper.deleteById(id);
    }

    private void validateScoreRuleExists(Long id) {
        if (scoreRuleMapper.selectById(id) == null) {
            throw exception(SCORE_RULE_NOT_EXISTS);
        }
    }

    @Override
    public ScoreRuleBackDetailVO getScoreRule(Long id) {
        ScoreRuleDO scoreRuleDO = scoreRuleMapper.selectById(id);
        if (scoreRuleDO == null) {
            throw exception(SCORE_RULE_NOT_EXISTS);
        }
        ScoreRuleBackDetailVO scoreRuleBackDetailVO = new ScoreRuleBackDetailVO();
        BeanUtils.copyProperties(scoreRuleDO, scoreRuleBackDetailVO);
        setExtraVO(scoreRuleBackDetailVO);
        return scoreRuleBackDetailVO;
    }


    @Override
    public List<ScoreRuleBackVO> getScoreRuleList(Collection<Long> ids) {
        List<ScoreRuleDO> scoreRuleDOS = scoreRuleMapper.selectBatchIds(ids);
        return ScoreRuleConvert.INSTANCE.convertList(scoreRuleDOS);
    }

    @Override
    public PageResult<ScoreRuleBackVO> getScoreRulePage(ScoreRuleQueryVO query, PageVO page) {
        PageResult<ScoreRuleDO> pageResult = scoreRuleMapper.selectPage(page, query);
        return ScoreRuleConvert.INSTANCE.convertPage(pageResult);
    }

    /**
     * 设置返回VO额外信息
     *
     * @param scoreRuleBackDetailVO VO
     */
    private void setExtraVO(ScoreRuleBackDetailVO scoreRuleBackDetailVO) {
        if (scoreRuleBackDetailVO.getType() == ScoreRuleTypeEnum.ORDER_V.getValue()) {
            scoreRuleBackDetailVO.setExtraOrderV(JSONUtil.toBean(scoreRuleBackDetailVO.getExtra(), ScoreRuleOrderVExtraVO.class));
        } else if (scoreRuleBackDetailVO.getType() == ScoreRuleTypeEnum.REGISTER.getValue()) {
            scoreRuleBackDetailVO.setExtraRegister(JSONUtil.toBean(scoreRuleBackDetailVO.getExtra(), ScoreRuleRegisterExtraVO.class));
        } else if (scoreRuleBackDetailVO.getType() == ScoreRuleTypeEnum.RECOMMEND.getValue()) {
            scoreRuleBackDetailVO.setExtraRecommend(JSONUtil.toBean(scoreRuleBackDetailVO.getExtra(), ScoreRulerRecommendExtraVO.class));
        } else if (scoreRuleBackDetailVO.getType() == ScoreRuleTypeEnum.SHARE.getValue()) {
            scoreRuleBackDetailVO.setExtraShare(JSONUtil.toBean(scoreRuleBackDetailVO.getExtra(), ScoreRuleShareExtraVO.class));
        } else {
            throw exception(SCORE_RULE_NOT_EXISTS);
        }
    }

    @Override
    public List<ScoreRuleDO> getScoreRuleList(ScoreRuleQueryVO query) {
        return scoreRuleMapper.selectList(query);
    }

    @Override
    public void updateStatus(ScoreRuleStatusReqVO scoreRuleStatusReqVO) {
        ScoreRuleDO scoreRuleDO = scoreRuleMapper.selectById(scoreRuleStatusReqVO.getId());
        if (scoreRuleDO == null) {
            throw exception(SCORE_RULE_NOT_EXISTS);
        }
        Integer oldStatus = scoreRuleDO.getStatus();
        ScoreRuleDO upScoreRuleDO = new ScoreRuleDO();
        upScoreRuleDO.setId(scoreRuleStatusReqVO.getId());
        if (oldStatus == ScoreRuleStatusEnum.ENABLED.getValue() && scoreRuleStatusReqVO.getStatus() == ScoreRuleStatusEnum.CLOSED.getValue()) {
            upScoreRuleDO.setStatus(scoreRuleStatusReqVO.getStatus());
        } else if (oldStatus == ScoreRuleStatusEnum.DISABLED.getValue() && scoreRuleStatusReqVO.getStatus() == ScoreRuleStatusEnum.ENABLED.getValue()) {
            //如果是启用，校验同一个规则下，有效期内有没有重复的规则设置，分享不校验
            if (scoreRuleDO.getType() != ScoreRuleTypeEnum.SHARE.getValue()) {
                LambdaQuery<ScoreRuleDO> scoreRuleDOLambdaQuery = new LambdaQuery<>();
                scoreRuleDOLambdaQuery.eq(ScoreRuleDO::getStatus, ScoreRuleStatusEnum.ENABLED.getValue())
                        .eq(ScoreRuleDO::getType, scoreRuleDO.getType())
                        .eq(ScoreRuleDO::getStatus, ScoreRuleStatusEnum.ENABLED.getValue());
                Long count = scoreRuleMapper.selectCount(scoreRuleDOLambdaQuery);
                if (count > 0) {
                    throw exception(SCORE_RULE_FIELD_ERROR);
                }
            }
            upScoreRuleDO.setStatus(scoreRuleStatusReqVO.getStatus());
        } else {
            throw exception(SCORE_RULE_UPDATE_ERROR);
        }
        scoreRuleMapper.updateById(upScoreRuleDO);
        scoreRuleRedisDao.deleteEnableScoreRule(scoreRuleDO.getType());
    }

    @Override
    public Long copyScoreRule(Long id) {
        ScoreRuleDO scoreRuleDO = scoreRuleMapper.selectById(id);
        if (scoreRuleDO == null) {
            throw exception(SCORE_RULE_NOT_EXISTS);
        }
        scoreRuleDO.setStatus(ScoreRuleStatusEnum.DISABLED.getValue());
        scoreRuleDO.setId(null);
        scoreRuleDO.setCreateTime(null);
        scoreRuleDO.setUpdateTime(null);
        scoreRuleDO.setUpdater(null);
        scoreRuleMapper.insert(scoreRuleDO);
        return scoreRuleDO.getId();
    }

    @Override
    public void delayScoreRule(ScoreDelayReqVO delayReqVO) {
        ScoreRuleDO scoreRuleDO = scoreRuleMapper.selectById(delayReqVO.getId());
        if (scoreRuleDO == null) {
            throw exception(SCORE_RULE_NOT_EXISTS);
        }
        if (scoreRuleDO.getStatus() != ScoreRuleStatusEnum.ENABLED.getValue()) {
            throw exception(SCORE_RULE_UPDATE_ERROR);
        }
        Instant now = Instant.now();
        //结束时间不能小于当前时间
        if (!delayReqVO.getEndTime().toInstant().isAfter(now) || !delayReqVO.getEndTime().toInstant().isAfter(scoreRuleDO.getStartTime().toInstant())) {
            throw exception(SCORE_RULE_UPDATE_ERROR);
        }
        ScoreRuleDO upScoreRuleDO = new ScoreRuleDO();
        upScoreRuleDO.setId(delayReqVO.getId());
        upScoreRuleDO.setEndTime(delayReqVO.getEndTime());
        scoreRuleMapper.updateById(upScoreRuleDO);
    }

    @Override
    public ScoreRuleDO getEnabledOrderVScoreRuleByTransportType(Integer transportType) {
        if (transportType != TransportTypeEnum.OCEAN_LCL.getValue() && transportType != TransportTypeEnum.SPECIAL_LINE_AIR_FREIGHT.getValue()) {
            throw exception(SCORE_RULE_FIELD_ERROR);
        }
        LambdaQuery<ScoreRuleDO> lambdaQuery = new LambdaQuery<>();
        lambdaQuery.eq(ScoreRuleDO::getStatus, ScoreRuleStatusEnum.ENABLED.getValue())
                .eq(ScoreRuleDO::getType, ScoreRuleTypeEnum.ORDER_V.getValue());
        List<ScoreRuleDO> scoreRuleDOS = scoreRuleMapper.selectList(lambdaQuery);
        if (!CollectionUtil.isEmpty(scoreRuleDOS)) {
            for (ScoreRuleDO scoreRuleDO : scoreRuleDOS) {
                if (Objects.equals(JSONUtil.toBean(scoreRuleDO.getExtra(), ScoreRuleOrderVExtraVO.class).getTransportType(), transportType)) {
                    return scoreRuleDO;
                }
            }
        }
        return null;
    }

    @Override
    public ScoreRuleDO getEnableScoreRuleByType(ScoreRuleTypeEnum scoreRuleType) {
        ScoreRuleDO enableScoreRule;
        enableScoreRule = scoreRuleRedisDao.getEnableScoreRule(scoreRuleType);
        if (enableScoreRule == null) {
            enableScoreRule = selectOne(new LambdaQuery<ScoreRuleDO>()
                    .eq(ScoreRuleDO::getType, scoreRuleType.getValue())
                    .eq(ScoreRuleDO::getStatus, ScoreRuleStatusEnum.ENABLED.getValue()));
            if (enableScoreRule == null) {
                return null;
            }
            // 校验一下活动开始结束时间
            if (!enableScoreRule.getEndTime().after(new Date()) || !enableScoreRule.getStartTime().before(new Date())) {
                return null;
            }
            scoreRuleRedisDao.setEnableScoreRule(enableScoreRule);
        } else {
            if (!enableScoreRule.getEndTime().after(new Date()) || !enableScoreRule.getStartTime().before(new Date())) {
                return null;
            }
        }
        return enableScoreRule;
    }

    @Override
    public ScoreRuleDO getEnableScoreRuleByType(ScoreRuleTypeEnum scoreRuleType, TransportTypeEnum transportType) {
        ScoreRuleDO enableScoreRule;
        switch (scoreRuleType) {
            case ORDER_V:
                enableScoreRule = scoreRuleRedisDao.getEnableScoreRule(scoreRuleType, transportType);
                if (enableScoreRule == null) {
                    enableScoreRule = getEnabledOrderVScoreRuleByTransportType(transportType.getValue());
                    if (enableScoreRule == null) {
                        return null;
                    }
                    if (!enableScoreRule.getEndTime().after(new Date()) || !enableScoreRule.getStartTime().before(new Date())) {
                        return null;
                    }
                    scoreRuleRedisDao.setEnableScoreRule(enableScoreRule);
                } else {
                    if (!enableScoreRule.getEndTime().after(new Date()) || !enableScoreRule.getStartTime().before(new Date())) {
                        return null;
                    }
                }
                return enableScoreRule;
            case RECOMMEND:
            case REGISTER:
                return getEnableScoreRuleByType(scoreRuleType);
        }
        return null;
    }
}
